
/***************************************************************************
 *   This file is part of Aspect, a simple PEC tool.                       *
 *                                                                         *
 *   Copyright (C) 2006-2007 by Wolfgang Hoffmann <woho@woho.de>           *
 *                                                                         *
 *   This program is free software, licensed under the GPL v2.             *
 *   See the file COPYING for more details.                                *
 ***************************************************************************/


#include "chart.h"
#include "log.h"

#include <QtGui>


Chart::Chart(QWidget *pParent)
:   QGraphicsView(pParent)
    {
    m_pRenderTimer = new QTimer(this);
    m_pRenderTimer->setSingleShot(true);
    m_bNeedSceneRender = false;
    connect(m_pRenderTimer, SIGNAL(timeout()), this, SLOT(performSceneRender()));
    }


void Chart::resizeEvent(QResizeEvent *pEvent)
    {
    QGraphicsView::resizeEvent(pEvent);
    requestSceneRender();
    }


void Chart::requestSceneRender()
    {
    // do some simple load balancing:
    // if last scene render is younger than 50 msecs,
    // delay next scene render by at least 50 msecs
    m_bNeedSceneRender = true;
    if (m_pRenderTimer->isActive())
        m_pRenderTimer->stop();
    else
        performSceneRender();
    m_pRenderTimer->start(50);
    }


void Chart::performSceneRender()
    {
    fitInView(sceneRect());
    }


