
/***************************************************************************
 *   This file is part of Aspect, a simple PEC tool.                       *
 *                                                                         *
 *   Copyright (C) 2007 by Wolfgang Hoffmann <woho@woho.de>                *
 *                                                                         *
 *   This program is free software, licensed under the GPL v2.             *
 *   See the file COPYING for more details.                                *
 ***************************************************************************/

#if 0
#include "data.h"
#include "log.h"
#include "mcuserial.h"
#include "mcucmd.h"

#include <QtGui>
#include <math.h>


MainWindow::MainWindow()
:   QMainWindow()
    {
    m_pUi = new Ui::MainWindow;
    m_pUi->setupUi(this);

    m_pPeData = new QList<PeValue>;
    m_pPeriod = new QList<Period>;
    m_pPeSorted = new QVector<QList<double> >(256);
    m_pPeCombined = new QVector<double>(256);
    m_pPeProcessed = new QVector<double>(256);
    m_pPecTable = new QVector<int>(256);
    m_pPeModel = new QStandardItemModel;
    m_pPeProcessedModel = new QStandardItemModel;
    m_pPecModel = new QStandardItemModel;
    m_pUi->viewPe->setModel(m_pPeModel);
    m_pUi->viewPeProcessed->setModel(m_pPeProcessedModel);
    m_pUi->viewPec->setModel(m_pPecModel);

    m_pPeChart = new QGraphicsScene(this);
    m_pUi->chartPe->setScene(m_pPeChart);

    connect(m_pPeModel, SIGNAL(itemChanged(QStandardItem *)), this, SLOT(changePeItem(QStandardItem *)));
    connect(m_pPecModel, SIGNAL(itemChanged(QStandardItem *)), this, SLOT(changePecItem(QStandardItem *)));

    // todo: implement functionality
    m_pUi->pushButtonPecCalc->setEnabled(false);
    m_pUi->toggleChartPeResidual->setChecked(false);
    m_pUi->toggleChartPeResidual->setEnabled(false);

    // woho's personal defaults to ease testing :-)
    // pixel scale: measured: 1.37, calculated: 1.31 (5.6µm @ 880mm: atan2(5.6e-6, 880e-3) / M_PI * 180. * 3600.)
    m_pUi->aquiPixelScale->setValue(1.31);
    m_pUi->aquiStarDec->setValue(15. + 14./60. + 38./3600.);
    m_pUi->aquiDrift->setValue(-.91);
    m_pUi->wormTime->setValue(55.);
    m_pUi->wormPos->setValue(91);
    openPeFile(QString::fromUtf8("/media/sdb8/pub/webcam/pe4x.dat"));
    openPecFile(QString::fromUtf8("/media/sdb8/pub/webcam/littlefoot/pec_20061015a.csv"));
    resize(960, 640);
    }


MainWindow::~MainWindow()
    {
    delete m_pUi;
    delete m_pPeData;
    delete m_pPeriod;
    delete m_pPeSorted;
    delete m_pPeCombined;
    delete m_pPeProcessed;
    delete m_pPecTable;
    }


void MainWindow::on_actionQuit_triggered()
    {
    LOG(4, "MainWindow::on_actionQuit_triggered");
    close();
    }


void MainWindow::on_actionAbout_Aspect_triggered()
    {
    LOG(4, "MainWindow::on_actionAbout_Aspect_triggered");
    QMessageBox::about(this,
        tr("About Aspect ..."),
        tr("<h3>Aspect</h3><p>"
           "Version: <b>0.2</b> (03 Jan 2007)<br>"
           "Autor: Wolfgang Hoffmann (<a href=\"mailto:woho@woho.de\">woho@woho.de</a>)</p>"
           "<p>Aspect is a simple PEC tool, allowing to "
           "view, edit and calculate periodic error correction "
           "tables for the <a href=\"http://littlefoot.rajiva.de/\">LittleFoot</a> "
           "astronomical mount stepper controler.</p>"
           "<p>Copyright (C) 2006-2007 Wolfgang Hoffmann.<br>"
           "This program is free software, licensed under the GPL v2.</p>"
           "<p>This program is distributed in the hope that it will be useful, "
           "but WITHOUT ANY WARRANTY; without even the implied warranty of "
           "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the "
           "GNU General Public License for more details.</p>"));
    }


void MainWindow::on_actionAbout_Qt_triggered()
    {
    LOG(4, "MainWindow::on_actionAbout_Qt_triggered");
    QApplication::aboutQt();
    }


void MainWindow::on_actionOpen_PE_Data_triggered()
    {
    LOG(4, "MainWindow::on_actionOpen_PE_Data_triggered");
    QFileDialog oFileDialog(this);
    oFileDialog.setAcceptMode(QFileDialog::AcceptOpen);
    oFileDialog.setFileMode(QFileDialog::ExistingFile);
    oFileDialog.setViewMode(QFileDialog::Detail);
    oFileDialog.setWindowTitle(tr("Open PE Data File"));
    oFileDialog.setDirectory("/home/woho/pub/webcam/");
    QStringList qslFilters;
    qslFilters << tr("MCU Control Plus Format (*.csv)");
    qslFilters << tr("IRIS Format (*.dat)");
    qslFilters << tr("All Files (*)");
    oFileDialog.setFilters(qslFilters);

    QStringList qslFileNames;
    if (oFileDialog.exec())
        qslFileNames = oFileDialog.selectedFiles();
    if (qslFileNames.size() > 0)
        {
        QString qsFileName = qslFileNames.at(0);
        LOG(3, "%s", qsFileName.toLocal8Bit().constData());
        openPeFile(qsFileName);
        }
    }


void MainWindow::on_actionOpen_PEC_Table_triggered()
    {
    LOG(4, "MainWindow::on_actionOpen_PEC_Table_triggered");
    QFileDialog oFileDialog(this);
    oFileDialog.setAcceptMode(QFileDialog::AcceptOpen);
    oFileDialog.setFileMode(QFileDialog::ExistingFile);
    oFileDialog.setViewMode(QFileDialog::Detail);
    oFileDialog.setWindowTitle(tr("Open PEC Table File"));
    oFileDialog.setDirectory("/home/woho/pub/webcam/littlefoot/");
    QStringList qslFilters;
    qslFilters << tr("PEC File (*.csv)");
    qslFilters << tr("All Files (*)");
    oFileDialog.setFilters(qslFilters);

    QStringList qslFileNames;
    if (oFileDialog.exec())
        qslFileNames = oFileDialog.selectedFiles();
    if (qslFileNames.size() > 0)
        {
        QString qsFileName = qslFileNames.at(0);
        LOG(3, "%s", qsFileName.toLocal8Bit().constData());
        openPecFile(qsFileName);
        }
    }


void MainWindow::on_actionSave_PEC_Table_triggered()
    {
    LOG(4, "MainWindow::on_actionSave_PEC_Table_triggered");
    QFileDialog oFileDialog(this);
    oFileDialog.setAcceptMode(QFileDialog::AcceptSave);
    oFileDialog.setFileMode(QFileDialog::AnyFile);
    oFileDialog.setViewMode(QFileDialog::Detail);
    oFileDialog.setWindowTitle(tr("Save PEC Table As File"));
    oFileDialog.setDirectory("/home/woho/pub/webcam/littlefoot/");
    QStringList qslFilters;
    qslFilters << tr("PEC File (*.csv)");
    qslFilters << tr("All Files (*)");
    oFileDialog.setFilters(qslFilters);

    QStringList qslFileNames;
    if (oFileDialog.exec())
        qslFileNames = oFileDialog.selectedFiles();
    if (qslFileNames.size() > 0)
        {
        QString qsFileName = qslFileNames.at(0);
        LOG(3, "%s", qsFileName.toLocal8Bit().constData());
        savePecFile(qsFileName);
        }
    }


void MainWindow::on_actionGet_PEC_Table_triggered()
    {
    LOG(4, "MainWindow::on_actionGet_PEC_Table_triggered");
    getPecTable();
    }


void MainWindow::on_actionPut_PEC_Table_triggered()
    {
    LOG(4, "MainWindow::on_actionPut_PEC_Table_triggered");
    putPecTable();
    }


void MainWindow::on_actionCalculate_PEC_triggered()
    {
    LOG(4, "MainWindow::on_actionCalculate_PEC_triggered");
    // todo
    }


void MainWindow::on_aquiPixelScale_valueChanged(double dfScale)
    {
    LOG(4, "MainWindow::on_aquiPixelScale_valueChanged");
    Q_UNUSED(dfScale)
    recalcPeData();
    }


void MainWindow::on_aquiStarDec_valueChanged(double dfDeclination)
    {
    LOG(4, "MainWindow::on_aquiStarDec_valueChanged");
    Q_UNUSED(dfDeclination)
    recalcPeData();
    }


void MainWindow::on_aquiDrift_valueChanged(double dfDrift)
    {
    LOG(4, "MainWindow::on_aquiDrift_valueChanged");
    Q_UNUSED(dfDrift)
    recalcPeData();
    }


void MainWindow::on_wormTime_valueChanged(double dfTime)
    {
    LOG(4, "MainWindow::on_wormTime_valueChanged");
    Q_UNUSED(dfTime)
    recalcPeData();
    }


void MainWindow::on_wormPos_valueChanged(int nPosition)
    {
    LOG(4, "MainWindow::on_wormPos_valueChanged");
    Q_UNUSED(nPosition)
    recalcPeData();
    }


void MainWindow::on_wormSpeed_activated(int nSpeed)
    {
    LOG(4, "MainWindow::on_wormSpeed_activated");
    Q_UNUSED(nSpeed)
    recalcPeData();
    }


void MainWindow::on_wormHemisphere_activated(int nHemisphere)
    {
    LOG(4, "MainWindow::on_wormHemisphere_activated");
    Q_UNUSED(nHemisphere)
    recalcPeData();
    }


void MainWindow::on_wormRotation_activated(int nRotation)
    {
    LOG(4, "MainWindow::on_wormRotation_activated");
    Q_UNUSED(nRotation)
    recalcPeData();
    }


void MainWindow::on_wormGearTeeth_valueChanged(int nTeeth)
    {
    LOG(4, "MainWindow::on_wormGearTeeth_valueChanged");
    Q_UNUSED(nTeeth)
    recalcPeData();
    }


void MainWindow::on_peProcCombining_activated(int nMethod)
    {
    LOG(4, "MainWindow::on_peProcCombining_activated");
    Q_UNUSED(nMethod)
    recalcPeProcessed();
    }


void MainWindow::on_peProcSmoothWidth_valueChanged(int nWidth)
    {
    LOG(4, "MainWindow::on_peProcSmoothWidth_valueChanged");
    Q_UNUSED(nWidth)
    recalcPeProcessed();
    }


void MainWindow::on_toggleChartPeRaw_toggled(bool bOn)
    {
    LOG(4, "MainWindow::on_toggleChartPeRaw_toggled");
    Q_UNUSED(bOn)
    recalcPeChart();
    }


void MainWindow::on_toggleChartPeProcessed_toggled(bool bOn)
    {
    LOG(4, "MainWindow::on_toggleChartPeProcessed_toggled");
    Q_UNUSED(bOn)
    recalcPeChart();
    }


void MainWindow::on_toggleChartPec_toggled(bool bOn)
    {
    LOG(4, "MainWindow::on_toggleChartPec_toggled");
    Q_UNUSED(bOn)
    recalcPeChart();
    }


void MainWindow::on_toggleChartPeResidual_toggled(bool bOn)
    {
    LOG(4, "MainWindow::on_toggleChartPeResidual_toggled");
    Q_UNUSED(bOn)
    recalcPeChart();
    }



void MainWindow::openPeFile(const QString &qsFileName)
    {
    LOG(4, "MainWindow::openPeFile");
    m_pUi->lineEditPeFile->setText(qsFileName);
    m_pPeModel->clear();

    m_pPeData->clear();
    QFile oFile(qsFileName);
    oFile.open(QIODevice::ReadOnly | QIODevice::Text);

    // decide on file format
    QTextStream oStream(&oFile);
    QString qsFirstLine = oStream.readLine();
    oStream.seek(0);
    if (qsFirstLine.left(6) == QString::fromUtf8("Angle,"))
        {
        // Doug Dieter's MCU Control Plus format:
        // header: camera angle, pixel scale (including star declination)
        // data: time (hour:min:sec), pos x (float), pox y (float)
        /*
Angle,0.15
Scale,1.155168
16:00:04,087.847,201.00,88.00
16:00:05,087.133,201.00,88.00
...
        */
        m_bCalculateWormPos = false;
        while (!oStream.atEnd())
            {
            QString qsLine = oStream.readLine();
            LOG(5, "    %s", qsLine.toLocal8Bit().constData());
            QStringList qslEntryStrings = qsLine.split(",");
            PeValue oEntry;
            oEntry.m_dfTime = 0.// (qslEntryStrings.size() > 0)? qslEntryStrings.at(0).toDouble(): -1;
            oEntry.m_dfStarPos = (qslEntryStrings.size() > 1)? qslEntryStrings.at(1).toDouble(): 0;
            m_pPeData->append(oEntry);
            }
        }
    else
        {
        // IRIS format:
        // no header
        // data: time (sec.msec), pixel pos (float)
        /*
0.000 0.000
2.013 1.792
4.016 -0.065
...
        */
        m_bCalculateWormPos = true;
        while (!oStream.atEnd())
            {
            QString qsLine = oStream.readLine();
            LOG(5, "    %s", qsLine.toLocal8Bit().constData());
            QStringList qslEntryStrings = qsLine.split(" ");
            PeValue oEntry;
            oEntry.m_dfTime = (qslEntryStrings.size() > 0)? qslEntryStrings.at(0).toDouble(): -1;
            oEntry.m_dfStarPos = (qslEntryStrings.size() > 1)? qslEntryStrings.at(1).toDouble(): 0;
            m_pPeData->append(oEntry);
            }
        }
    oFile.close();

    m_pPeModel->setColumnCount(4);
    m_pPeModel->setRowCount(m_pPeData->size());

    QStringList qslHeaders;
    qslHeaders << tr("Time");
    qslHeaders << tr("Star Position");
    qslHeaders << tr("Worm Position");
    qslHeaders << tr("PE");
    m_pPeModel->setHorizontalHeaderLabels(qslHeaders);

    for (int nRow = 0; nRow < m_pPeData->size(); nRow++)
        {
        QStandardItem *pItem;
        pItem = new QStandardItem(QString::number(m_pPeData->at(nRow).m_dfTime, 'f', 3));
        pItem->setEditable(false);
        pItem->setSelectable(false);
        m_pPeModel->setItem(nRow, 0, pItem);
        pItem = new QStandardItem(QString::number(m_pPeData->at(nRow).m_dfStarPos));
        pItem->setEditable(true);
        pItem->setSelectable(true);
        m_pPeModel->setItem(nRow, 1, pItem);
        }
    oFile.close();
    recalcPeData();
    }


void MainWindow::openPecFile(const QString &qsFileName)
    {
    LOG(4, "MainWindow::openPecFile");
    m_pPecModel->clear();

    QFile oFile(qsFileName);
    oFile.open(QIODevice::ReadOnly | QIODevice::Text);
    QTextStream oStream(&oFile);
    QString qsHeader = oStream.readLine();
    if (qsHeader != QString::fromUtf8("Factor, Table, ReqSpeed, IsSpeed, MinSpeed, MaxSpeed"))
        {
        LOG(0, "MainWindow::openPecFile(): wrong file format");
        return;
        }
    for (int nRow = 0; nRow < 256; nRow++)
        {
        (*m_pPecTable)[nRow] = 0;
        if (oStream.atEnd())
            continue;
        QString qsLine = oStream.readLine();
        LOG(5, "    %s", qsLine.toLocal8Bit().constData());
        QStringList qslEntryStrings = qsLine.split(",");
        int nFactor = (qslEntryStrings.size() > 0)? qslEntryStrings.at(0).toInt(): 1;
        int nTable = (qslEntryStrings.size() > 1)? qslEntryStrings.at(1).toInt(): -1;
        //int nReqSpeed = (qslEntryStrings.size() > 2)? qslEntryStrings.at(2).toInt(): -1;
        //int nIsSpeed = (qslEntryStrings.size() > 3)? qslEntryStrings.at(3).toInt(): -1;
        //int nMinSpeed = (qslEntryStrings.size() > 4)? qslEntryStrings.at(4).toInt(): -1;
        //int nMaxSpeed = (qslEntryStrings.size() > 5)? qslEntryStrings.at(5).toInt(): -1;
        if (nRow == 0)
            m_nPecFactor = nFactor;
        (*m_pPecTable)[nRow] = nTable;
        }
    oFile.close();

    m_pPecModel->setColumnCount(2);
    m_pPecModel->setRowCount(256);

    QStringList qslHeaders;
    qslHeaders << tr("Worm Position");
    qslHeaders << tr("PEC");
    m_pPecModel->setHorizontalHeaderLabels(qslHeaders);

    for (int nRow = 0; nRow < 256; nRow++)
        {
        QStandardItem *pItem;
        pItem = new QStandardItem(QString::number(nRow));
        pItem->setEditable(false);
        pItem->setSelectable(false);
        m_pPecModel->setItem(nRow, 0, pItem);
        pItem = new QStandardItem(QString::number(m_pPecTable->at(nRow)));
        pItem->setEditable(true);
        pItem->setSelectable(true);
        m_pPecModel->setItem(nRow, 1, pItem);
        }
    oFile.close();
    recalcPeChart();
    }


void MainWindow::savePecFile(const QString &qsFileName)
    {
    LOG(4, "MainWindow::savePecFile");

    QFile oFile(qsFileName);
    oFile.open(QIODevice::WriteOnly | QIODevice::Text);
    QTextStream oStream(&oFile);
    oStream << QString::fromUtf8("Factor, Table, ReqSpeed, IsSpeed, MinSpeed, MaxSpeed\n");
    int nReqSpeed = 0;
    for (int nRow = 0; nRow < 256; nRow++)
        {
        int nFactor = 1;
        int nTable = m_pPecTable->at(nRow);
        if (nTable == 1)
            nReqSpeed += 1;
        else if (nTable == 2)
            nReqSpeed -= 1;
        else if (nTable == 3)
            nReqSpeed = 0;
        int nIsSpeed = nReqSpeed;
        int nMinSpeed = -47;
        int nMaxSpeed = 47;
        if (nIsSpeed < nMinSpeed)
            nIsSpeed = nMinSpeed;
        if (nIsSpeed > nMaxSpeed)
            nIsSpeed = nMaxSpeed;
        oStream << QString::fromUtf8("%1, %2, %3, %4, %5, %6\n")
            .arg(nFactor).arg(nTable)
            .arg(nReqSpeed).arg(nIsSpeed)
            .arg(nMinSpeed).arg(nMaxSpeed);
        }
    oStream << QString::fromUtf8("\n");
    oFile.close();
    }


void MainWindow::getPecTable()
    {
    LOG(4, "MainWindow::getPecTable");
    m_pPecModel->clear();

    QProgressDialog oProgress(tr("Get PEC table from MCU ..."), tr("Cancel"), 0, 256, this);
    oProgress.setWindowModality(Qt::WindowModal);

    McuSerial oPort;
    oPort.open(0);
    McuDataPecTable oPec;
    oPec.mcuGetFactor(oPort);
    for (int nRow = 0; nRow < 256; nRow++)
        {
        oProgress.setValue(nRow);
        qApp->processEvents();

        if (oProgress.wasCanceled())
            break;
        oPec.mcuGetValue(oPort, nRow);
        }
    oProgress.setValue(256);
    oPort.close();
    m_nPecFactor = oPec.factor();
    for (int nRow = 0; nRow < 256; nRow++)
        {
        int nTable = oPec.value(nRow);
        //int nReqSpeed = (qslEntryStrings.size() > 2)? qslEntryStrings.at(2).toInt(): -1;
        //int nIsSpeed = (qslEntryStrings.size() > 3)? qslEntryStrings.at(3).toInt(): -1;
        //int nMinSpeed = (qslEntryStrings.size() > 4)? qslEntryStrings.at(4).toInt(): -1;
        //int nMaxSpeed = (qslEntryStrings.size() > 5)? qslEntryStrings.at(5).toInt(): -1;
        (*m_pPecTable)[nRow] = nTable;
        }

    m_pPecModel->setColumnCount(2);
    m_pPecModel->setRowCount(256);

    QStringList qslHeaders;
    qslHeaders << tr("Worm Position");
    qslHeaders << tr("PEC");
    m_pPecModel->setHorizontalHeaderLabels(qslHeaders);

    for (int nRow = 0; nRow < 256; nRow++)
        {
        QStandardItem *pItem;
        pItem = new QStandardItem(QString::number(nRow));
        pItem->setEditable(false);
        pItem->setSelectable(false);
        m_pPecModel->setItem(nRow, 0, pItem);
        pItem = new QStandardItem(QString::number(m_pPecTable->at(nRow)));
        pItem->setEditable(true);
        pItem->setSelectable(true);
        m_pPecModel->setItem(nRow, 1, pItem);
        }
    recalcPeChart();
    }


void MainWindow::putPecTable()
    {
    LOG(4, "MainWindow::putPecTable");
    }


void MainWindow::changePeItem(QStandardItem *pItem)
    {
    LOG(5, "MainWindow::changePeItem");
    if ((pItem == 0) || (pItem->column() != 1))
        return;
    int nRow = pItem->row();
    if ((nRow < 0) || (nRow >= m_pPeData->size()))
        return;
    double dfStarPos = pItem->text().toDouble();
    if (m_pPeData->at(nRow).m_dfStarPos == dfStarPos)
        return;
    (*m_pPeData)[nRow].m_dfStarPos = dfStarPos;
    recalcPeData();
    }


void MainWindow::changePecItem(QStandardItem *pItem)
    {
    LOG(5, "MainWindow::changePecItem");
    if ((pItem == 0) || (pItem->column() != 1))
        return;
    int nRow = pItem->row();
    if ((nRow < 0) || (nRow >= 256))
        return;
    int nPec = pItem->text().toInt();
    if (m_pPecTable->at(nRow) == nPec)
        return;
    (*m_pPecTable)[nRow] = nPec;
    recalcPeChart();
    }


void MainWindow::recalcPeData()
    {
    LOG(4, "MainWindow::recalcPeData");
    double dfWormGearPeriod =
        (m_pUi->wormSpeed->currentIndex() == 0)? 86164.: 86400.;    // 0=Sidereal, 1=Solar
    double dfWormDirection =
        ((m_pUi->wormHemisphere->currentIndex() == 0)? -1.: 1.) *   // 0=Northern, 1=Southern
        ((m_pUi->wormRotation->currentIndex() == 0)? 1.: -1.);      // 0=Normal, 1=Reversed
    int nWormGearTeeth = m_pUi->wormGearTeeth->value();
    double dfTime = m_pUi->wormTime->value();
    double dfPos = m_pUi->wormPos->value();
    double dfWormPeriod = dfWormGearPeriod / (double)nWormGearTeeth;
    double dfWormSpeed = 256. / dfWormPeriod * dfWormDirection;
    double dfWormStart = dfPos - dfTime * dfWormSpeed;
    dfWormStart -= floor(dfWormStart / 256.) * 256.;
    double dfDrift = m_pUi->aquiDrift->value() / 60.;
    double dfScale = m_pUi->aquiPixelScale->value() * cos(m_pUi->aquiStarDec->value() * M_PI / 180.);

    for (int nRow = 0; nRow < m_pPeData->size(); nRow++)
        {
        dfTime = m_pPeData->at(nRow).m_dfTime;
        dfPos = dfWormStart + dfTime * dfWormSpeed;
        dfPos -= floor(dfPos / 256.) * 256.;
        (*m_pPeData)[nRow].m_dfWormPos = dfPos;
        (*m_pPeData)[nRow].m_dfPe = (m_pPeData->at(nRow).m_dfStarPos - dfTime * dfDrift) * dfScale;
        QStandardItem *pItem;
        pItem = new QStandardItem(QString::number(floor(m_pPeData->at(nRow).m_dfWormPos), 'f', 0));
        pItem->setEditable(false);
        pItem->setSelectable(false);
        m_pPeModel->setItem(nRow, 2, pItem);
        pItem = new QStandardItem(QString::number(m_pPeData->at(nRow).m_dfPe));
        pItem->setEditable(false);
        pItem->setSelectable(false);
        m_pPeModel->setItem(nRow, 3, pItem);
        }

    recalcPeriods();
    }


void MainWindow::recalcPeriods()
    {
    LOG(4, "MainWindow::recalcPeriods");

    m_pPeriod->clear();

    Period oPeriod;
    oPeriod.m_nRowBeg = -1;
    double dfWormOld = 1000.;
    for (int nRow = 0; nRow < m_pPeData->size(); nRow++)
        {
        oPeriod.m_nRowEnd = nRow;
        double dfWormPos = m_pPeData->at(nRow).m_dfWormPos;
        // a leap of WormPos indicates end of current period
        if (fabs(dfWormPos - dfWormOld) > 128.)
            {
            if (oPeriod.m_nRowBeg >= 0)
                m_pPeriod->append(oPeriod);
            oPeriod.m_nRowBeg = nRow;
            oPeriod.m_dfPeMin = 1e10;
            oPeriod.m_dfPeMax = -1e10;
            }
        double dfPe = m_pPeData->at(nRow).m_dfPe;
        if (oPeriod.m_dfPeMin > dfPe)
            oPeriod.m_dfPeMin = dfPe;
        if (oPeriod.m_dfPeMax < dfPe)
            oPeriod.m_dfPeMax = dfPe;
        dfWormOld = dfWormPos;
        }
    if (oPeriod.m_nRowBeg >= 0)
        m_pPeriod->append(oPeriod);

    for (int nRow = 0; nRow < 256; nRow++)
        (*m_pPeSorted)[nRow].clear();

    for (int nRow = 0; nRow < m_pPeData->size(); nRow++)
        {
        double dfWormPos = m_pPeData->at(nRow).m_dfWormPos;
        double dfPe = m_pPeData->at(nRow).m_dfPe;
        int nWormPos = (int)floor(dfWormPos);
        if ((nWormPos >= 0) && (nWormPos < 256))
            (*m_pPeSorted)[nWormPos].append(dfPe);
        }

    recalcPeProcessed();
    }


void MainWindow::recalcPeProcessed()
    {
    LOG(4, "MainWindow::recalcPeProcessed");

    if (m_pUi->peProcCombining->currentIndex() == 0)                // 0=Average, 1=Median
        {
        for (int nRow = 0; nRow < 256; nRow++)
            {
            // Average
            double dfPe = 0.;
            int nSampleNum = m_pPeSorted->at(nRow).size();
            for (int nSample = 0; nSample < nSampleNum; nSample++)
                dfPe += m_pPeSorted->at(nRow).at(nSample);
            if (nSampleNum > 0)
                dfPe /= (double)nSampleNum;
            (*m_pPeCombined)[nRow] = dfPe;
            }
        }
    else
        {
        for (int nRow = 0; nRow < 256; nRow++)
            {
            // Median
            double dfPe = 0.;
            int nSampleNum = m_pPeSorted->at(nRow).size();
            if (nSampleNum > 0)
                {
                QList<double> oList(m_pPeSorted->at(nRow));
                qSort(oList);
                dfPe = oList.at(nSampleNum / 2);
                }
            (*m_pPeCombined)[nRow] = dfPe;
            }
        }

    int nSmoothWidth = m_pUi->peProcSmoothWidth->value();
    for (int nRow = 0; nRow < 256; nRow++)
        {
        // Average
        double dfPe = 0.;
        for (int nSmooth = 0; nSmooth < nSmoothWidth; nSmooth++)
            dfPe += m_pPeCombined->at((nRow + (nSmooth - nSmoothWidth / 2) + 256) % 256);
        (*m_pPeProcessed)[nRow] = dfPe / (double)nSmoothWidth;
        }

    m_pPeProcessedModel->setColumnCount(2);
    m_pPeProcessedModel->setRowCount(256);

    QStringList qslHeaders;
    qslHeaders << tr("Worm Position");
    qslHeaders << tr("PE");
    m_pPeProcessedModel->setHorizontalHeaderLabels(qslHeaders);

    for (int nRow = 0; nRow < 256; nRow++)
        {
        QStandardItem *pItem;
        pItem = new QStandardItem(QString::number(nRow));
        pItem->setEditable(false);
        pItem->setSelectable(false);
        m_pPeProcessedModel->setItem(nRow, 0, pItem);
        pItem = new QStandardItem(QString::number(m_pPeProcessed->at(nRow)));
        pItem->setEditable(false);
        pItem->setSelectable(false);
        m_pPeProcessedModel->setItem(nRow, 1, pItem);
        }

    recalcPeChart();
    }


void MainWindow::recalcPeChart()
    {
    LOG(4, "MainWindow::recalcPeChart");
    QList<QGraphicsItem *> oItemList = m_pPeChart->items();
    for (int nItem = 0; nItem < oItemList.size(); nItem++)
        {
        QGraphicsItem *pItem = oItemList.at(nItem);
        m_pPeChart->removeItem(pItem);
        delete pItem;
        }

    bool bChartPeRaw = m_pUi->toggleChartPeRaw->isChecked();
    bool bChartPeProcessed = m_pUi->toggleChartPeProcessed->isChecked();
    bool bChartPec = m_pUi->toggleChartPec->isChecked();
    bool bChartPeResidual = m_pUi->toggleChartPeResidual->isChecked();
    bool bChartPe = bChartPeRaw || bChartPeProcessed || bChartPeResidual;

    int nPeMin = 1000;
    int nPeMax = -1000;
    for (int nPeriod = 0; nPeriod < m_pPeriod->size(); nPeriod++)
        {
        if (nPeMin > (int)floor(m_pPeriod->at(nPeriod).m_dfPeMin))
            nPeMin = (int)floor(m_pPeriod->at(nPeriod).m_dfPeMin);
        if (nPeMax < (int)ceil(m_pPeriod->at(nPeriod).m_dfPeMax))
            nPeMax = (int)ceil(m_pPeriod->at(nPeriod).m_dfPeMax);
        }
    int nPeMid = (nPeMin + nPeMax) / 2 / 5 * 5;

    int nPecMin = 0;
    int nPecMax = 0;
    int nReqSpeed = 0;
    static int anAcc[4] = { 0, 1, -1, 2 };
    for (int nRow = 0; nRow < 256; nRow++)
        {
        int nTable = m_pPecTable->at(nRow);
        int nAcc = anAcc[nTable % 4];
        nReqSpeed = (nTable == 3)? 0: nReqSpeed + nAcc;
        if (nPecMin > nReqSpeed)
            nPecMin = nReqSpeed;
        if (nPecMax < nReqSpeed)
            nPecMax = nReqSpeed;
        }
    int nPecMid = (nPecMin + nPecMax) / 2 / 5 * 5;

    int nXMin = 0;
    int nXMax = 256;
    int nYMin = 1000;
    int nYMax = -1000;
    if (bChartPe)
        {
        if (nYMin > nPeMin - nPeMid)
            nYMin = nPeMin - nPeMid;
        if (nYMax < nPeMax - nPeMid)
            nYMax = nPeMax - nPeMid;
        }
    if (bChartPec)
        {
        if (nYMin > nPecMin - nPecMid)
            nYMin = nPecMin - nPecMid;
        if (nYMax < nPecMax - nPecMid)
            nYMax = nPecMax - nPecMid;
        }
    double dfXScale = 500. / (double)(nXMax - nXMin + 1);
    double dfYScale = -500. / (double)(nYMax - nYMin + 1);
    double dfY2Scale = -10.;

    QPainterPath oPathGrid;
    QPainterPath oPathGridMajor;
    for (int nX = nXMin; nX <= nXMax; nX += 8)
        {
        QPainterPath &rPath = ((nX % 32) == 0)? oPathGridMajor: oPathGrid;
        rPath.moveTo(dfXScale * (double)nX, dfYScale * (double)nYMin);
        rPath.lineTo(dfXScale * (double)nX, dfYScale * (double)nYMax);
        if (nX % 32 == 0)
            {
            QGraphicsTextItem *pTextItem = m_pPeChart->addText(QString::number(nX));
            pTextItem->setPos(dfXScale * (double)nX - pTextItem->boundingRect().width() / 2,
                dfYScale * (double)nYMin + 2);
            }
        }
    for (int nY = nYMin; nY <= nYMax; nY++)
        {
        QPainterPath &rPath = ((nY % 5) == 0)? oPathGridMajor: oPathGrid;
        rPath.moveTo(dfXScale * (double)nXMin, dfYScale * (double)nY);
        rPath.lineTo(dfXScale * (double)nXMax, dfYScale * (double)nY);
        if (nY % 5 == 0)
            {
            if (bChartPe)
                {
                QGraphicsTextItem *pTextItem = m_pPeChart->addText(QString::number(nY + nPeMid));
                pTextItem->setPos(dfXScale * (double)nXMin - pTextItem->boundingRect().width() - 2,
                    dfYScale * (double)nY - pTextItem->boundingRect().height() / 2);
                }
            if (bChartPec)
                {
                QGraphicsTextItem *pTextItem = m_pPeChart->addText(QString::number(nY + nPecMid));
                pTextItem->setPos(dfXScale * (double)nXMax + 2,
                    dfYScale * (double)nY - pTextItem->boundingRect().height() / 2);
                }
            }
        }
    oPathGrid.moveTo(0., 0.);
    oPathGrid.closeSubpath();
    oPathGridMajor.moveTo(0., 0.);
    oPathGridMajor.closeSubpath();
    QGraphicsPathItem *pItem;
    pItem = m_pPeChart->addPath(oPathGrid, QPen(Qt::lightGray));
    pItem->setZValue(0.);
    pItem = m_pPeChart->addPath(oPathGridMajor, QPen(Qt::gray));
    pItem->setZValue(1.);

    if (bChartPeRaw)
        {
        for (int nPeriod = 0; nPeriod < m_pPeriod->size(); nPeriod++)
            {
            int nRowBeg = m_pPeriod->at(nPeriod).m_nRowBeg;
            int nRowEnd = m_pPeriod->at(nPeriod).m_nRowEnd;
            LOG(4, "MainWindow::recalcPeChart: period %d: [%d..%d[", nPeriod, nRowBeg, nRowEnd);
            QPainterPath oPath;
            for (int nRow = nRowBeg; nRow < nRowEnd; nRow++)
                {
                double dfX = m_pPeData->at(nRow).m_dfWormPos;
                double dfY = m_pPeData->at(nRow).m_dfPe;
                if (nRow == nRowBeg)
                    oPath.moveTo(dfXScale * dfX, dfYScale * (dfY - (double)nPeMid));
                else
                    oPath.lineTo(dfXScale * dfX, dfYScale * (dfY - (double)nPeMid));
                }
            oPath.moveTo(0., 0.);
            oPath.closeSubpath();
            pItem = m_pPeChart->addPath(oPath);
            pItem->setZValue(2.);
            }
        }

    if (bChartPeProcessed)
        {
        QPainterPath oPath;
        for (int nRow = 0; nRow <= 256; nRow++)
            {
            double dfX = (double)nRow;
            double dfY = m_pPeProcessed->at(nRow % 256);
            if (nRow == 0)
                oPath.moveTo(dfXScale * dfX, dfYScale * (dfY - (double)nPeMid));
            else
                oPath.lineTo(dfXScale * dfX, dfYScale * (dfY - (double)nPeMid));
            }
        oPath.moveTo(0., 0.);
        oPath.closeSubpath();
        pItem = m_pPeChart->addPath(oPath, QPen(Qt::red, 2));
        pItem->setZValue(3.);
        }

    if (bChartPec)
        {
        QPainterPath oPathPec;
        QPainterPath oPathSpeed;
        oPathPec.moveTo(0., dfYScale * (double)nYMin + dfY2Scale * (double)(0 - 4));
        nReqSpeed = 0;
        for (int nRow = 0; nRow < 256; nRow++)
            {
            int nTable = m_pPecTable->at(nRow);
            int nAcc = anAcc[nTable % 4];
            nReqSpeed = (nTable == 3)? 0: nReqSpeed + nAcc;
            oPathPec.lineTo(dfXScale * (double)nRow,
                dfYScale * (double)nYMin + dfY2Scale * (double)(nAcc - 4));
            oPathPec.lineTo(dfXScale * (double)(nRow+1),
                dfYScale * (double)nYMin + dfY2Scale * (double)(nAcc - 4));
            oPathSpeed.lineTo(dfXScale * ((double)nRow + .5), dfYScale * (double)(nReqSpeed - nPecMid));
            }
        oPathPec.moveTo(0., 0.);
        oPathPec.closeSubpath();
        oPathSpeed.moveTo(0., 0.);
        oPathSpeed.closeSubpath();
        pItem = m_pPeChart->addPath(oPathPec, QPen(Qt::darkGreen));
        pItem->setZValue(2.);
        pItem = m_pPeChart->addPath(oPathSpeed, QPen(Qt::darkGreen));
        pItem->setZValue(2.);
        }

    if (bChartPeResidual)
        {
        }

    m_pPeChart->setSceneRect(m_pPeChart->itemsBoundingRect());
    }

#endif
