
/***************************************************************************
 *   This file is part of Aspect, a simple PEC tool.                       *
 *                                                                         *
 *   Copyright (C) 2006-2007 by Wolfgang Hoffmann <woho@woho.de>           *
 *                                                                         *
 *   This program is free software, licensed under the GPL v2.             *
 *   See the file COPYING for more details.                                *
 ***************************************************************************/


#ifndef ASPECT_MAINWINDOW_H
#define ASPECT_MAINWINDOW_H


#include <QMainWindow>
#include <QList>
#include <QVector>

class QGraphicsScene;
class QStandardItem;
class QStandardItemModel;
class IPeRawData;
class PePureSample;
class Period;
class McuSerial;

namespace Ui { class MainWindow; };


class MainWindow: public QMainWindow
    {
    Q_OBJECT

public:
    MainWindow();
    ~MainWindow();

public slots:
    void on_actionQuit_triggered();
    void on_actionAbout_Aspect_triggered();
    void on_actionAbout_Qt_triggered();

    void on_actionOpen_PE_Data_triggered();
    void on_widgetPeMcu_dataPeChanged(IPeRawData *pData);
    void on_widgetPeIris_dataPeChanged(IPeRawData *pData);

    void on_peProcDrift_valueChanged(double dfDrift);
    void on_peProcCombining_activated(int nMethod);
    void on_peProcSmoothWidth_valueChanged(int nWidth);

    void on_actionOpen_PEC_Table_triggered();
    void on_actionSave_PEC_Table_triggered();
    void on_actionGet_PEC_Table_triggered();
    void on_actionPut_PEC_Table_triggered();
    void on_actionCalculate_PEC_triggered();
    void on_pecFactor_valueChanged(int nFactor);

    void on_mcuStart_clicked();
    void on_mcuStop_clicked();
    void mcuTimer();

    void on_toggleChartPeRaw_toggled(bool bOn);
    void on_toggleChartPeProcessed_toggled(bool bOn);
    void on_toggleChartPec_toggled(bool bOn);
    void on_toggleChartPeResidual_toggled(bool bOn);

private:
    void openPeFile(const QString &qsFileName);
    void openPecFile(const QString &qsFileName);
    void savePecFile(const QString &qsFileName);
    void getPecTable();
    void putPecTable();

private slots:
    void changePecItem(QStandardItem *pItem);

private:
    void recalcPeData();
    void recalcPeriods();
    void recalcPeProcessed();
    void recalcPeChart();

private:
    Ui::MainWindow *m_pUi;
    IPeRawData *m_pPeRaw;
    QList<PePureSample> m_lstPePure;
    QList<Period> m_lstPeriod;
    QVector<QList<double> > m_aPeSorted;
    QVector<double> m_aPeCombined;
    QVector<double> m_aPeProcessed;
    int m_nPecFactor;
    QVector<int> m_aPecTable;
    QList<PePureSample> m_lstPecProfile;
    QStandardItemModel *m_pPeProcessedModel;
    QStandardItemModel *m_pPecModel;
    QStandardItemModel *m_pPecProfileModel;
    QGraphicsScene *m_pPeChart;
    QTimer *m_pMcuTimer;
    McuSerial *m_pMcuPort;
    };


#endif // ASPECT_MAINWINDOW_H

