
/***************************************************************************
 *   This file is part of Aspect, a simple PEC tool.                       *
 *                                                                         *
 *   Copyright (C) 2006-2007 by Wolfgang Hoffmann <woho@woho.de>           *
 *                                                                         *
 *   This program is free software, licensed under the GPL v2.             *
 *   See the file COPYING for more details.                                *
 ***************************************************************************/


#ifndef ASPECT_MCUCMD_H
#define ASPECT_MCUCMD_H


#include <QByteArray>
#include <QString>
#include <QTime>

class McuSerial;


class IMcuCmd
    {
public:
    virtual ~IMcuCmd() {};
private:
    virtual bool flush() const { return true; };
    virtual QByteArray command() const = 0;
    virtual int receive(const QByteArray &aAnswerFragment) = 0;
    virtual QString name() const = 0;
friend class McuSerial;
    };


class McuDataFirmware
    {
public:
    bool mcuGet(McuSerial &rPort);
    QString firmware() const { return m_qsFirmware; };
private:
    QString m_qsFirmware;
    };

class McuDataState
    {
public:
    bool mcuGet(McuSerial &rPort);
    QString backlash() const;
    QString guideRate() const;
    QString paddle8Rate() const;
    QString paddle16Rate() const;
    QString gotoRate() const;
    QString orientation() const;
    QString trackingRate() const;
    int gearboxReduction() const;
    int wormRatio() const;
    int wormTeeth() const;
private:
    QByteArray m_aState;
    };

class McuDataPecState
    {
public:
    bool mcuGet(McuSerial &rPort);
    QString state() const;
    double wormPos() const;
    QTime time() const { return m_oTimeBeg; };
private:
    QByteArray m_aState;
    QTime m_oTimeBeg;
    //QTime m_oTimeEnd;
    };

class McuDataPecTable
    {
public:
    McuDataPecTable();
    bool mcuGetFactor(McuSerial &rPort);
    bool mcuGetValue(McuSerial &rPort, int nRow);
    bool mcuSetFactor(McuSerial &rPort) const;
    bool mcuSetValue(McuSerial &rPort, int nRow) const;
    bool mcuSave(McuSerial &rPort) const;
    int factor() const { return m_nFactor; };
    int value(int nRow) const { return ((nRow < 0) || (nRow >= c_nRowNum))? 0: m_anValue[nRow]; };
    bool setFactor(int nFactor);
    bool setValue(int nRow, int nValue);
private:
    static const int c_nRowNum = 256;
    int m_nFactor;
    int m_anValue[c_nRowNum];
    };


#endif // ASPECT_MCUCMD_H

